@@ -20,7 +20,19 @@ setClass("GRanges",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Validity.
+### "update" method
+###
+
+### Having update() redirect to BiocGenerics:::replaceSlots() on GRanges
+### objects makes all the methods for GenomicRanges objects defined in
+### R/GenomicRanges-class.R work on GRanges objects.
+setMethod("update", "GRanges",
+    function(object, ...) BiocGenerics:::replaceSlots(object, ...)
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Validity
 ###
 
 .valid.GRanges.ranges <- function(x)
@@ -47,7 +59,7 @@ setValidity2("GRanges", .valid.GRanges)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Constructor.
+### Constructor
 ###
 
 ### Hidden constructor shared with other GRanges-like objects.
@@ -137,7 +149,7 @@ newGRanges <- function(class,
     if (ncol(mcols) == 0L) {
         mcols <- mcols(ranges)
         if (is.null(mcols))
-            mcols <- new("DataFrame", nrows=length(seqnames))
+            mcols <- S4Vectors:::make_zero_col_DataFrame(length(seqnames))
     }
     if (!is.null(mcols(ranges)))
         mcols(ranges) <- NULL
@@ -186,9 +198,25 @@ setMethod("updateObject", "GRanges",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Coercion.
+### Coercion
 ###
 
+setMethod("granges", "GenomicRanges",
+    function(x, use.mcols=FALSE)
+    {
+        if (!isTRUEorFALSE(use.mcols))
+            stop("'use.mcols' must be TRUE or FALSE")
+        ans <- GRanges(seqnames(x), ranges(x), strand(x), seqinfo=seqinfo(x))
+        if (use.mcols)
+            mcols(ans) <- cbind(extraColumnSlotsAsDF(x), mcols(x))
+        ans
+    }
+)
+
+setAs("GenomicRanges", "GRanges",
+    function(from) granges(from, use.mcols=TRUE)
+)
+
 .from_character_to_GRanges <- function(from)
 {
     stopifnot(is.character(from))
@@ -202,8 +230,8 @@ setMethod("updateObject", "GRanges",
         "end positions). Strand can be \"+\", \"-\", \"*\", or missing."
     )
     split0 <- CharacterList(strsplit(from, ":", fixed=TRUE))
-    split0_eltlens <- elementLengths(split0)
-    if (S4Vectors:::anyMissingOrOutside(split0_eltlens, 2L, 3L))
+    split0_eltNROWS <- elementNROWS(split0)
+    if (S4Vectors:::anyMissingOrOutside(split0_eltNROWS, 2L, 3L))
         stop(error_msg)
     ans_strand <- as.character(ptail(split0, n=-2L))
     ans_strand[is.na(ans_strand)] <- "*"
@@ -214,8 +242,8 @@ setMethod("updateObject", "GRanges",
     ## a digit (ignoring and removing the spaces in between if any).
     ranges <- sub("([[:digit:]])[[:space:]]*-", "\\1..", ranges)
     split2 <- CharacterList(strsplit(ranges, "..", fixed=TRUE))
-    split2_eltlens <- elementLengths(split2)
-    if (!all(split2_eltlens == 2L))
+    split2_eltNROWS <- elementNROWS(split2)
+    if (!all(split2_eltNROWS == 2L))
         stop(error_msg)
     ans_start <- as.integer(phead(split2, n=1L))
     ans_end <- as.integer(ptail(split2, n=1L))
@@ -284,7 +312,7 @@ setAs("Seqinfo", "GRanges", .fromSeqinfoToGRanges)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Slot getters and setters.
+### Slot getters and setters
 ###
 
 setMethod("seqnames", "GRanges", function(x) x@seqnames)
@@ -302,3 +330,108 @@ setMethod("ranges", "GRanges",
 setMethod("strand", "GRanges", function(x) x@strand)
 setMethod("seqinfo", "GRanges", function(x) x@seqinfo)
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Subsetting
+###
+
+setMethod("extractROWS", "GRanges",
+    function(x, i)
+    {
+        if (missing(i) || !is(i, "Ranges"))
+            i <- normalizeSingleBracketSubscript(i, x)
+        ans_seqnames <- extractROWS(seqnames(x), i)
+        ans_ranges <- extractROWS(ranges(x), i)
+        ans_strand <- extractROWS(strand(x), i)
+        ans_mcols <- extractROWS(mcols(x), i)
+        ans_ecs <- lapply(extraColumnSlots(x), extractROWS, i)
+        BiocGenerics:::replaceSlots(x, seqnames=ans_seqnames,
+                                       ranges=ans_ranges,
+                                       strand=ans_strand,
+                                       elementMetadata=ans_mcols,
+                                       .slotList=ans_ecs,
+                                       check=FALSE)
+    }
+)
+
+setMethod("replaceROWS", "GRanges",
+    function(x, i, value)
+    {
+        if (missing(i) || !is(i, "Ranges"))
+            i <- normalizeSingleBracketSubscript(i, x)
+        seqinfo(x) <- merge(seqinfo(x), seqinfo(value))
+        ans_seqnames <- replaceROWS(seqnames(x), i, seqnames(value))
+        ans_ranges <- replaceROWS(ranges(x), i, ranges(value))
+        ans_strand <- replaceROWS(strand(x), i, strand(value))
+        ans_mcols <- replaceROWS(mcols(x), i, mcols(value))
+        ans_ecs_names <- extraColumnSlotNames(x)
+        ans_necs <- length(ans_ecs_names)
+        if (ans_necs == 0L) {
+            ans_ecs <- NULL
+        } else {
+            value_ecs_names <- extraColumnSlotNames(value)
+            if (!identical(value_ecs_names[seq_len(ans_necs)],
+                           ans_ecs_names))
+                stop("'value' can have more extra column slots but not less")
+            ans_ecs <- extraColumnSlotsAsDF(x)
+            value_ecs <- extraColumnSlotsAsDF(value)
+            ans_ecs <- replaceROWS(ans_ecs, i, value_ecs[seq_len(ans_necs)])
+        }
+        BiocGenerics:::replaceSlots(x, seqnames=ans_seqnames,
+                                       ranges=ans_ranges,
+                                       strand=ans_strand,
+                                       elementMetadata=ans_mcols,
+                                       .slotList=as.list(ans_ecs))
+    }
+)
+
+### TODO: Refactor to use replaceROWS(). This will make the code much simpler
+### and avoid a lot of duplication with the above "replaceROWS" method.
+setReplaceMethod("[", "GRanges",
+    function(x, i, j, ..., value)
+    {
+        if (!is(value, "GenomicRanges"))
+            stop("replacement value must be a GenomicRanges object")
+        seqinfo(x) <- merge(seqinfo(x), seqinfo(value))
+        seqnames <- seqnames(x)
+        ranges <- ranges(x)
+        strand <- strand(x)
+        ans_mcols <- mcols(x, FALSE)
+        value_ecs <- extraColumnSlotsAsDF(value)
+        x_ecs <- extraColumnSlotsAsDF(x)
+        new_ecs <- value_ecs[!names(value_ecs) %in% names(x_ecs)]
+        ecs_to_replace <- intersect(names(value_ecs), names(x_ecs))
+        if (missing(i)) {
+            seqnames[] <- seqnames(value)
+            ranges[] <- ranges(value)
+            strand[] <- strand(value)
+            if (missing(j))
+                ans_mcols[ , ] <- mcols(value, FALSE)
+            else
+                ans_mcols[ , j] <- mcols(value, FALSE)
+            if (length(new_ecs) > 0L)
+                ans_mcols[names(new_ecs)] <- new_ecs
+            x_ecs[ecs_to_replace] <- value_ecs[ecs_to_replace]
+        } else {
+            i <- extractROWS(setNames(seq_along(x), names(x)), i)
+            seqnames[i] <- seqnames(value)
+            ranges[i] <- ranges(value)
+            strand[i] <- strand(value)
+            if (missing(j))
+                ans_mcols[i, ] <- mcols(value, FALSE)
+            else
+                ans_mcols[i, j] <- mcols(value, FALSE)
+            if (length(new_ecs) > 0L)
+                ans_mcols[i, names(new_ecs)] <- DataFrame(new_ecs)
+            if (length(ecs_to_replace) > 0L) {
+              x_ecs[i, ecs_to_replace] <- value_ecs[ecs_to_replace]
+            }
+        }
+        BiocGenerics:::replaceSlots(x, seqnames=seqnames,
+                                       ranges=ranges,
+                                       strand=strand,
+                                       elementMetadata=ans_mcols,
+                                       .slotList=as.list(x_ecs))
+    }
+)
+
