@@ -77,12 +77,12 @@ makeGRangesListFromFeatureFragments <- function(seqnames=Rle(factor()),
 {
     fragmentStarts <- normargListOfIntegers(fragmentStarts, sep,
                                             "fragmentStarts")
-    nfrag_per_feature <- elementLengths(fragmentStarts)
+    nfrag_per_feature <- elementNROWS(fragmentStarts)
     start <- unlist(fragmentStarts, recursive=FALSE, use.names=FALSE)
 
     fragmentEnds <- normargListOfIntegers(fragmentEnds, sep,
                                           "fragmentEnds")
-    nend_per_elt <- elementLengths(fragmentEnds)
+    nend_per_elt <- elementNROWS(fragmentEnds)
     if (length(nend_per_elt) != 0L) {
         if (length(nfrag_per_feature) == 0L)
             nfrag_per_feature <- nend_per_elt
@@ -94,7 +94,7 @@ makeGRangesListFromFeatureFragments <- function(seqnames=Rle(factor()),
 
     fragmentWidths <- normargListOfIntegers(fragmentWidths, sep,
                                             "fragmentWidths")
-    nwidth_per_elt <- elementLengths(fragmentWidths)
+    nwidth_per_elt <- elementNROWS(fragmentWidths)
     if (length(nwidth_per_elt) != 0L) {
         if (length(nfrag_per_feature) == 0L)
             nfrag_per_feature <- nwidth_per_elt
@@ -161,9 +161,9 @@ setMethod("seqnames", "GRangesList",
 replaceSeqnamesList <- function(x, value)
 {
     if (!is(value, "AtomicList") ||
-        !identical(elementLengths(x), elementLengths(value)))
+        !identical(elementNROWS(x), elementNROWS(value)))
         stop("replacement 'value' is not an AtomicList with the same ",
-             "elementLengths as 'x'")
+             "elementNROWS as 'x'")
     value <- unlist(value, use.names = FALSE)
     if (!is(value, "Rle"))
         value <- Rle(factor(value))
@@ -194,9 +194,9 @@ setReplaceMethod("ranges", "GRangesList",
     function(x, value) 
     {
         if (!is(value, "RangesList") ||
-            !identical(elementLengths(x), elementLengths(value)))
+            !identical(elementNROWS(x), elementNROWS(value)))
             stop("replacement 'value' is not a RangesList with the same ",
-                 "elementLengths as 'x'")
+                 "elementNROWS as 'x'")
         ranges(x@unlistData) <- as(unlist(value, use.names = FALSE), "IRanges")
         x
     }
@@ -215,9 +215,9 @@ setReplaceMethod("start", "GRangesList",
     function(x, ..., value)
     {
         if (!is(value, "IntegerList") ||
-            !identical(elementLengths(x), elementLengths(value)))
+            !identical(elementNROWS(x), elementNROWS(value)))
             stop("replacement 'value' is not an IntegerList with the same ",
-                 "elementLengths as 'x'")
+                 "elementNROWS as 'x'")
         value <- unlist(value, use.names = FALSE)
         start(x@unlistData, ...) <- value
         x
@@ -237,9 +237,9 @@ setReplaceMethod("end", "GRangesList",
     function(x, ..., value)
     {
         if (!is(value, "IntegerList") ||
-            !identical(elementLengths(x), elementLengths(value)))
+            !identical(elementNROWS(x), elementNROWS(value)))
             stop("replacement 'value' is not an IntegerList with the same ",
-                 "elementLengths as 'x'")
+                 "elementNROWS as 'x'")
         value <- unlist(value, use.names = FALSE)
         end(x@unlistData, ...) <- value
         x
@@ -259,9 +259,9 @@ setReplaceMethod("width", "GRangesList",
     function(x, ..., value)
     {
         if (!is(value, "IntegerList") ||
-            !identical(elementLengths(x), elementLengths(value)))
+            !identical(elementNROWS(x), elementNROWS(value)))
             stop("replacement 'value' is not an IntegerList with the same ",
-                 "elementLengths as 'x'")
+                 "elementNROWS as 'x'")
         value <- unlist(value, use.names = FALSE)
         width(x@unlistData, ...) <- value
         x
@@ -280,9 +280,9 @@ setMethod("strand", "GRangesList",
 replaceStrandList <- function(x, value)
 {
     if (!is(value, "AtomicList") ||
-        !identical(elementLengths(x), elementLengths(value)))
+        !identical(elementNROWS(x), elementNROWS(value)))
         stop("replacement 'value' is not an AtomicList with the same ",
-             "elementLengths as 'x'")
+             "elementNROWS as 'x'")
     value <- unlist(value, use.names = FALSE)
     if (!is(value, "Rle"))
         value <- Rle(strand(value))
@@ -331,7 +331,7 @@ replaceElementMetadataList <-
         level <- match.arg(level)
     if (level == "between") {
         if (is.null(value))
-            value <- new("DataFrame", nrows = length(x))
+            value <- S4Vectors:::make_zero_col_DataFrame(length(x))
         else if (!is(value, "DataFrame"))
             value <- DataFrame(value)
         if (!is.null(rownames(value)))
@@ -346,12 +346,12 @@ replaceElementMetadataList <-
         x@elementMetadata <- value
     } else {
         if (is.null(value)) {
-            value <- new("DataFrame", nrows = length(x@unlistData))
+            value <- S4Vectors:::make_zero_col_DataFrame(length(x@unlistData))
         } else {
             if (!is(value, "SplitDataFrameList") ||
-                !identical(elementLengths(x), elementLengths(value))) {
+                !identical(elementNROWS(x), elementNROWS(value))) {
                 stop("replacement 'value' is not a SplitDataFrameList with ",
-                        "the same elementLengths as 'x'")
+                        "the same elementNROWS as 'x'")
             }
             value <- unlist(value, use.names = FALSE)
         }
@@ -373,7 +373,7 @@ replaceSeqinfoList <- function(x, new2old=NULL, force=FALSE, value)
                               seqlevels(value))
     if (length(dangling_seqlevels) != 0L) {
         dropme <- which(seqnames(x@unlistData) %in% dangling_seqlevels)
-        x <- x[-unique(togroup(x, j=dropme))]
+        x <- x[-unique(togroup(PartitioningByWidth(x), j=dropme))]
     }
     seqinfo(x@unlistData, new2old=new2old) <- value
     x
@@ -503,7 +503,7 @@ setMethod("relistToClass", "GRanges", function(x) "GRangesList")
 showList <- function(object, showFunction, print.classinfo)
 {
     k <- length(object)
-    cumsumN <- cumsum(elementLengths(object))
+    cumsumN <- cumsum(elementNROWS(object))
     N <- tail(cumsumN, 1)
     cat(class(object), " object of length ", k, ":\n", sep = "")
     if (k == 0L) {
@@ -564,7 +564,7 @@ showList <- function(object, showFunction, print.classinfo)
 
 setMethod("show", "GRangesList",
     function(object)
-        showList(object, showGenomicRanges, TRUE)
+        showList(object, show_GenomicRanges, TRUE)
 )
 
 
@@ -583,7 +583,7 @@ setMethod("show", "GRangesList",
 deconstructGRLintoGR <- function(x, expand.levels=FALSE)
 {
     ans <- x@unlistData
-    f1 <- rep.int(seq_len(length(x)), elementLengths(x))
+    f1 <- rep.int(seq_len(length(x)), elementNROWS(x))
     f2 <- as.integer(seqnames(ans))
     f12 <- paste(f1, f2, sep="|")
 
@@ -593,12 +593,12 @@ deconstructGRLintoGR <- function(x, expand.levels=FALSE)
         i1 <- rep(seq_len(length(x)), each=x_nlev)
         i2 <- rep.int(seq_len(x_nlev), length(x))
     } else {
-        oo <- S4Vectors:::orderIntegerPairs(f1, f2)
+        oo <- orderIntegerPairs(f1, f2)
         of1 <- f1[oo]
         of2 <- f2[oo]
-        ## TODO: Add "presorted" method to S4Vectors:::duplicatedIntegerPairs()
-        ## for when the 2 input vectors are already sorted.
-        notdups <- !S4Vectors:::duplicatedIntegerPairs(of1, of2)
+        ## TODO: Support 'method="presorted"' in duplicatedIntegerPairs() for
+        ## when the 2 input vectors are already sorted.
+        notdups <- !duplicatedIntegerPairs(of1, of2)
         i1 <- of1[notdups]
         i2 <- of2[notdups]
     }
