@@ -18,7 +18,7 @@ setClassUnion("GenomicRangesORmissing", c("GenomicRanges", "missing"))
 
 ### The code in this file will work out-of-the-box on 'x' as long as
 ### seqnames(x), ranges(x), strand(x), seqlengths(x), seqinfo(),
-### update(x) and clone(x) are defined.
+### and update(x) are defined.
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -50,11 +50,6 @@ setGeneric("extraColumnSlotNames",
 
 setMethod("extraColumnSlotNames", "ANY", function(x) character())
 
-setMethod("parallelVectorNames", "GenomicRanges", function(x) {
-  colnames(as.data.frame(new(class(x))))
-})
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### 2 low-level helper functions to deal with out-of-bound ranges.
 ###
@@ -130,12 +125,16 @@ make_out_of_bound_warning_msg <- function(x, idx, suggest.trim)
     c(pbs1, pbs2)
 }
 
+### Used in GenomicAlignments.
 .valid.GenomicRanges.seqnames <- function(x)
 {
-    if (!is.factor(runValue(seqnames(x))))
-        return("'seqnames' should be a 'factor' Rle")
-    if (S4Vectors:::anyMissing(runValue(seqnames(x))))
-        return("'seqnames' contains missing values")
+    x_seqnames <- seqnames(x)
+    if (!is(x_seqnames, "Rle") || !is.factor(runValue(x_seqnames)))
+        return("'seqnames(x)' must be a 'factor' Rle")
+    if (!is.null(names(x_seqnames)))
+        return("'seqnames(x)' must be a 'factor' Rle with no names")
+    if (S4Vectors:::anyMissing(runValue(x_seqnames)))
+        return("'seqnames(x)' contains missing values")
     NULL
 }
 
@@ -328,18 +327,6 @@ setAs("Seqinfo", "RangesList",
     function(from) as(as(from, "GenomicRanges"), "RangesList")
 )
 
-setMethod("granges", "GenomicRanges",
-    function(x, use.mcols=FALSE)
-    {
-        if (!isTRUEorFALSE(use.mcols))
-            stop("'use.mcols' must be TRUE or FALSE")
-        ans <- GRanges(seqnames(x), ranges(x), strand(x), seqinfo=seqinfo(x))
-        if (use.mcols)
-            mcols(ans) <- cbind(extraColumnSlotsAsDF(x), mcols(x))
-        ans
-    }
-)
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Setters.
@@ -353,23 +340,23 @@ setReplaceMethod("names", "GenomicRanges",
     }
 )
 
+### Used in GenomicAlignments.
+.normalize_seqnames_replacement_value <- function(value, x)
+{
+    if (!is(value, "Rle"))
+        value <- Rle(value)
+    if (!is.factor(runValue(value)))
+        runValue(value) <- factor(runValue(value))
+    if (!identical(levels(value), seqlevels(x)))
+        stop("levels of supplied 'seqnames' must be ",
+             "identical to 'seqlevels(x)'")
+    S4Vectors:::V_recycle(value, x, x_what="value", skeleton_what="x")
+}
+
 setReplaceMethod("seqnames", "GenomicRanges",
     function(x, value)
     {
-        if (!is(value, "Rle"))
-            value <- Rle(value)
-        if (!is.factor(runValue(value)))
-            runValue(value) <- factor(runValue(value))
-        if (!identical(levels(value), seqlevels(x)))
-            stop("levels of supplied 'seqnames' must be ",
-                 "identical to 'seqlevels(x)'")
-        n <- length(x)
-        k <- length(value)
-        if (k != n) {
-            if ((k == 0L) || (k > n) || (n %% k != 0L))
-                stop(k, " elements in value to replace ", n, " elements")
-            value <- rep(value, length.out=n)
-        }
+        value <- .normalize_seqnames_replacement_value(value, x)
         update(x, seqnames=value)
     }
 )
@@ -377,16 +364,12 @@ setReplaceMethod("seqnames", "GenomicRanges",
 setReplaceMethod("ranges", "GenomicRanges",
     function(x, value)
     {
-        if (!is(value, "IRanges"))
+        if (class(value) != "IRanges")
             value <- as(value, "IRanges")
-        n <- length(x)
-        k <- length(value)
-        if (k != n) {
-            if ((k == 0L) || (k > n) || (n %% k != 0L))
-                stop(k, " elements in value to replace ", n, " elements")
-            value <- rep(value, length.out=n)
-        }
-        update(x, ranges=value, check=FALSE)
+        mcols(value) <- NULL
+        value <- S4Vectors:::V_recycle(value, x,
+                                       x_what="value", skeleton_what="x")
+        update(x, ranges=value)
     }
 )
 
@@ -418,18 +401,6 @@ setReplaceMethod("strand", "GenomicRanges",
     }
 )
 
-setReplaceMethod("elementMetadata", "GenomicRanges",
-    function(x, ..., value)
-    {
-        value <- normalizeMetadataColumnsReplacementValue(value, x)
-        x <- update(x, elementMetadata=value, check=FALSE)
-        msg <- .valid.GenomicRanges.mcols(x)
-        if (!is.null(msg))
-            stop(msg)
-        x
-    }
-)
-
 setReplaceMethod("seqinfo", "GenomicRanges",
     function(x, new2old=NULL, force=FALSE, value)
     {
@@ -498,11 +469,6 @@ setMethod("clone", "ANY",  # not exported
     }
 )
 
-setMethod("update", "GenomicRanges",
-          function(object, ...)
-          {
-            BiocGenerics:::replaceSlots(object, ...)
-          })
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Ranges methods.
@@ -541,25 +507,6 @@ setReplaceMethod("width", "GenomicRanges",
 ### Subsetting.
 ###
 
-setMethod("extractROWS", "GenomicRanges",
-    function(x, i)
-    {
-        if (missing(i) || !is(i, "Ranges"))
-            i <- normalizeSingleBracketSubscript(i, x)
-        ans_seqnames <- extractROWS(seqnames(x), i)
-        ans_ranges <- extractROWS(ranges(x), i)
-        ans_strand <- extractROWS(strand(x), i)
-        ans_mcols <- extractROWS(mcols(x), i)
-        ans_ecs <- lapply(extraColumnSlots(x), extractROWS, i)
-        clone(x, seqnames=ans_seqnames,
-                 ranges=ans_ranges,
-                 strand=ans_strand,
-                 elementMetadata=ans_mcols,
-                 .slotList=ans_ecs,
-                 check=FALSE)
-    }
-)
-
 ### Needed only because we want to support x[i, j] subsetting.
 setMethod("[", "GenomicRanges",
     function(x, i, j, ..., drop)
@@ -575,90 +522,11 @@ setMethod("[", "GenomicRanges",
     }
 )
 
-setMethod("replaceROWS", "GenomicRanges",
-    function(x, i, value)
-    {
-        if (missing(i) || !is(i, "Ranges"))
-            i <- normalizeSingleBracketSubscript(i, x)
-        seqinfo(x) <- merge(seqinfo(x), seqinfo(value))
-        ans_seqnames <- replaceROWS(seqnames(x), i, seqnames(value))
-        ans_ranges <- replaceROWS(ranges(x), i, ranges(value))
-        ans_strand <- replaceROWS(strand(x), i, strand(value))
-        ans_mcols <- replaceROWS(mcols(x), i, mcols(value))
-        ans_ecs_names <- extraColumnSlotNames(x)
-        ans_necs <- length(ans_ecs_names)
-        if (ans_necs == 0L) {
-            ans_ecs <- NULL
-        } else {
-            value_ecs_names <- extraColumnSlotNames(value)
-            if (!identical(value_ecs_names[seq_len(ans_necs)],
-                           ans_ecs_names))
-                stop("'value' can have more extra column slots but not less")
-            ans_ecs <- extraColumnSlotsAsDF(x)
-            value_ecs <- extraColumnSlotsAsDF(value)
-            ans_ecs <- replaceROWS(ans_ecs, i, value_ecs[seq_len(ans_necs)])
-        }
-        update(x, seqnames=ans_seqnames,
-                  ranges=ans_ranges,
-                  strand=ans_strand,
-                  elementMetadata=ans_mcols,
-                  .slotList=as.list(ans_ecs))
-    }
-)
-
-### TODO: Refactor to use replaceROWS(). This will make the code much simpler
-### and avoid a lot of duplication with the above "replaceROWS" method.
-setReplaceMethod("[", "GenomicRanges",
-    function(x, i, j, ..., value)
-    {
-        if (!is(value, "GenomicRanges"))
-            stop("replacement value must be a GenomicRanges object")
-        seqinfo(x) <- merge(seqinfo(x), seqinfo(value))
-        seqnames <- seqnames(x)
-        ranges <- ranges(x)
-        strand <- strand(x)
-        ans_mcols <- mcols(x, FALSE)
-        value_ecs <- extraColumnSlotsAsDF(value)
-        x_ecs <- extraColumnSlotsAsDF(x)
-        new_ecs <- value_ecs[!names(value_ecs) %in% names(x_ecs)]
-        ecs_to_replace <- intersect(names(value_ecs), names(x_ecs))
-        if (missing(i)) {
-            seqnames[] <- seqnames(value)
-            ranges[] <- ranges(value)
-            strand[] <- strand(value)
-            if (missing(j))
-                ans_mcols[ , ] <- mcols(value, FALSE)
-            else
-                ans_mcols[ , j] <- mcols(value, FALSE)
-            if (length(new_ecs) > 0L)
-                ans_mcols[names(new_ecs)] <- new_ecs
-            x_ecs[ecs_to_replace] <- value_ecs[ecs_to_replace]
-        } else {
-            i <- extractROWS(setNames(seq_along(x), names(x)), i)
-            seqnames[i] <- seqnames(value)
-            ranges[i] <- ranges(value)
-            strand[i] <- strand(value)
-            if (missing(j))
-                ans_mcols[i, ] <- mcols(value, FALSE)
-            else
-                ans_mcols[i, j] <- mcols(value, FALSE)
-            if (length(new_ecs) > 0L)
-                ans_mcols[i, names(new_ecs)] <- DataFrame(new_ecs)
-            if (length(ecs_to_replace) > 0L) {
-              x_ecs[i, ecs_to_replace] <- value_ecs[ecs_to_replace]
-            }
-        }
-        update(x, seqnames=seqnames, ranges=ranges,
-               strand=strand, elementMetadata=ans_mcols,
-               .slotList=as.list(x_ecs))
-    }
-)
-
 ### Subset a named list-like object *by* a GenomicRanges subscript.
 ### The returned object 'ans' is as follow:
 ###   (a) 'ans' is parallel to 'gr'.
 ###   (b) 'names(ans)' is identical to 'as.character(seqnames(gr))'.
-###   (c) 'elementLengths(ans)' is the same as 'width(gr)'.
+###   (c) 'elementNROWS(ans)' is the same as 'width(gr)'.
 ###   (d) 'class(ans)' is 'relistToClass(x[[1]])' e.g. CompressedRleList if
 ###       'x' is an RleList object, or DNAStringSet is 'x' is a DNAStringSet
 ###       object.
@@ -774,7 +642,7 @@ setReplaceMethod("$", "GenomicRanges",
     ans_ranges <- do.call(c, lapply(x, ranges))
     ans_strand <- do.call(c, lapply(x, strand))
     if (ignore.mcols) {
-        ans_mcols <- new("DataFrame", nrows=length(ans_ranges))
+        ans_mcols <- S4Vectors:::make_zero_col_DataFrame(length(ans_ranges))
     } else {
         ans_mcols <- do.call(rbind, lapply(x, mcols, FALSE))
     }
@@ -807,7 +675,7 @@ setMethod("c", "GenomicRanges",
 ### "show" method.
 ###
 
-.makeNakedMatFromGenomicRanges <- function(x)
+.make_naked_matrix_from_GenomicRanges <- function(x)
 {
     x_len <- length(x)
     x_mcols <- mcols(x)
@@ -828,8 +696,15 @@ setMethod("c", "GenomicRanges",
     ans
 }
 
-showGenomicRanges <- function(x, margin="",
-                              print.classinfo=FALSE, print.seqinfo=FALSE)
+### If 'x' is a GRanges object, 'coerce.internally.to.GRanges' has no effect.
+### If it's a GenomicRanges object that is not a GRanges object, then
+### show_GenomicRanges() will coerce it to a GRanges object unless
+### 'coerce.internally.to.GRanges' is set to FALSE. Use this if coercing 'x'
+### to GRanges is not supported or is too expensive but only if 'x' supports
+### head() and tail().
+show_GenomicRanges <- function(x, margin="",
+                               print.classinfo=FALSE, print.seqinfo=FALSE,
+                               coerce.internally.to.GRanges=TRUE)
 {
     x_class <- class(x)
     x_len <- length(x)
@@ -840,15 +715,15 @@ showGenomicRanges <- function(x, margin="",
         " and ",
         x_nmc, " metadata ", ifelse(x_nmc == 1L, "column", "columns"),
         ":\n", sep="")
-    ## S4Vectors:::makePrettyMatrixForCompactPrinting() assumes that 'x' is
-    ## subsettable but not all GenomicRanges objects are (and if they are,
-    ## subsetting them could be costly). However GRanges objects are assumed
-    ## to be subsettable so if 'x' is not one then we turn it into one (this
-    ## coercion is expected to work on any GenomicRanges object).
-    if (!is(x, "GRanges"))
-        x <- as(x, "GRanges")
-    out <- S4Vectors:::makePrettyMatrixForCompactPrinting(x,
-               .makeNakedMatFromGenomicRanges)
+    ## S4Vectors:::makePrettyMatrixForCompactPrinting() assumes that head()
+    ## and tail() work on 'xx'.
+    if (coerce.internally.to.GRanges) {
+        xx <- as(x, "GRanges", strict=FALSE)
+    } else {
+        xx <- x
+    }
+    out <- S4Vectors:::makePrettyMatrixForCompactPrinting(xx,
+                .make_naked_matrix_from_GenomicRanges)
     if (print.classinfo) {
         .COL2CLASS <- c(
             seqnames="Rle",
@@ -856,7 +731,7 @@ showGenomicRanges <- function(x, margin="",
             strand="Rle"
         )
         extraColumnNames <- extraColumnSlotNames(x)
-        .COL2CLASS <- c(.COL2CLASS, getSlots(class(x))[extraColumnNames])
+        .COL2CLASS <- c(.COL2CLASS, getSlots(x_class)[extraColumnNames])
         classinfo <-
             S4Vectors:::makeClassinfoRowForCompactPrinting(x, .COL2CLASS)
         ## A sanity check, but this should never happen!
@@ -877,14 +752,9 @@ showGenomicRanges <- function(x, margin="",
 
 setMethod("show", "GenomicRanges",
     function(object)
-        showGenomicRanges(object, margin="  ",
-                          print.classinfo=TRUE, print.seqinfo=TRUE)
+        show_GenomicRanges(object, margin="  ",
+                           print.classinfo=TRUE, print.seqinfo=TRUE)
 )
 
-setMethod("showAsCell", "GenomicRanges",
-          function(object) {
-            if (length(object) > 0L) {
-              paste0(seqnames(object), ":", strand(object), ":",
-                     showAsCell(ranges(object)))
-            } else character(0)
-          })
+setMethod("showAsCell", "GenomicRanges", function(object) as.character(object))
+
