@@ -17,7 +17,7 @@
     cil <- splitAsList(start(x), seqnames(x))  # CompressedIntegerList object
     ## The 4 lines below are equivalent to:
     ##   ans <- min(cil)
-    ##   ans[elementLengths(v) == 0L] <- NA_integer_
+    ##   ans[elementNROWS(v) == 0L] <- NA_integer_
     ## but much faster!
     ## TODO: Replace with the above, but only when the "min" method for
     ## CompressedIntegerList objects (implemented in the IRanges package)
@@ -36,7 +36,7 @@
     cil <- splitAsList(end(x), seqnames(x))  # CompressedIntegerList object
     ## The 4 lines below are equivalent to:
     ##   ans <- max(cil)
-    ##   ans[elementLengths(v) == 0L] <- NA_integer_
+    ##   ans[elementNROWS(v) == 0L] <- NA_integer_
     ## but much faster!
     ## TODO: Replace with the above, but only when the "max" method for
     ## CompressedIntegerList objects (implemented in the IRanges package)
@@ -56,7 +56,7 @@
 ###
 
 setMethod("union", c("GRanges", "GRanges"),
-    function(x, y, ignore.strand=FALSE, ...)
+    function(x, y, ignore.strand=FALSE)
     {
         if (!isTRUEorFALSE(ignore.strand))
             stop("'ignore.strand' must be TRUE or FALSE")
@@ -67,8 +67,23 @@ setMethod("union", c("GRanges", "GRanges"),
     }
 )
 
+### Equivalent to 'mendoapply(union, x, y)'.
+setMethod("union", c("GRangesList", "GRangesList"),
+    function(x, y, ...)
+    {
+        if (length(x) != length(y))
+            stop("'x' and 'y' must have the same length")
+        seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
+        seqlevels(y) <- seqlevels(x)
+        xgr <- deconstructGRLintoGR(x)
+        ygr <- deconstructGRLintoGR(y)
+        gr <- callGeneric(xgr, ygr, ...)
+        reconstructGRLfromGR(gr, x)
+    }
+)
+
 setMethod("intersect", c("GRanges", "GRanges"),
-    function(x, y, ignore.strand=FALSE, ...)
+    function(x, y, ignore.strand=FALSE)
     {
         if (!isTRUEorFALSE(ignore.strand))
             stop("'ignore.strand' must be TRUE or FALSE")
@@ -88,8 +103,23 @@ setMethod("intersect", c("GRanges", "GRanges"),
     }
 )
 
+### Equivalent to 'mendoapply(intersect, x, y)'.
+setMethod("intersect", c("GRangesList", "GRangesList"),
+    function(x, y, ...)
+    {
+        if (length(x) != length(y))
+            stop("'x' and 'y' must have the same length")
+        seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
+        seqlevels(y) <- seqlevels(x)
+        xgr <- deconstructGRLintoGR(x)
+        ygr <- deconstructGRLintoGR(y)
+        gr <- callGeneric(xgr, ygr, ...)
+        reconstructGRLfromGR(gr, x)
+    }
+)
+
 setMethod("setdiff", c("GRanges", "GRanges"),
-    function(x, y, ignore.strand=FALSE, ...)
+    function(x, y, ignore.strand=FALSE)
     {
         if (!isTRUEorFALSE(ignore.strand))
             stop("'ignore.strand' must be TRUE or FALSE")
@@ -109,6 +139,21 @@ setMethod("setdiff", c("GRanges", "GRanges"),
     }
 )
 
+### Equivalent to 'mendoapply(setdiff, x, y)'.
+setMethod("setdiff", c("GRangesList", "GRangesList"),
+    function(x, y, ...)
+    {
+        if (length(x) != length(y))
+            stop("'x' and 'y' must have the same length")
+        seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
+        seqlevels(y) <- seqlevels(x)
+        xgr <- deconstructGRLintoGR(x)
+        ygr <- deconstructGRLintoGR(y)
+        gr <- callGeneric(xgr, ygr, ...)
+        reconstructGRLfromGR(gr, x)
+    }
+)
+
 
 ### =========================================================================
 ### Parallel set operations
@@ -144,7 +189,7 @@ allCompatibleSeqnamesAndStrand <- function(x, y) {
 ###
 
 setMethod("punion", c("GRanges", "GRanges"),
-    function(x, y, fill.gap=FALSE, ignore.strand=FALSE, ...)
+    function(x, y, fill.gap=FALSE, ignore.strand=FALSE)
     {
         if (length(x) != length(y)) 
             stop("'x' and 'y' must have the same length")
@@ -172,7 +217,7 @@ setMethod("punion", c("GRanges", "GRanges"),
 ### Note that behaviour (b) could also be considered a valid candidate for
 ### a union,GRangesList,GRanges method (which we don't have at the moment).
 setMethod("punion", c("GRangesList", "GRanges"),
-    function(x, y, fill.gap=FALSE, ...)
+    function(x, y, fill.gap=FALSE)
     {
         n <- length(x)
         if (n != length(y)) 
@@ -181,17 +226,14 @@ setMethod("punion", c("GRangesList", "GRanges"),
         mcols(y) <- NULL
         ans <-
           split(c(x@unlistData, y), 
-                c(Rle(seq_len(n), elementLengths(x)), Rle(seq_len(n))))
+                c(Rle(seq_len(n), elementNROWS(x)), Rle(seq_len(n))))
         names(ans) <- names(x)
         ans
     }
 )
 
 setMethod("punion", c("GRanges", "GRangesList"),
-    function(x, y, fill.gap=FALSE, ...)
-    {
-        callGeneric(y, x)
-    }
+    function(x, y, ...) callGeneric(y, x, ...)
 )
 
 
@@ -307,7 +349,7 @@ setMethod("pintersect", c("GRanges", "GRanges"),
     if (!isTRUEorFALSE(drop.nohit.ranges))
         stop(wmsg("'drop.nohit.ranges' must be TRUE or FALSE"))
     unlisted_x <- unlist(x, use.names=FALSE)
-    y2 <- rep.int(y, elementLengths(x))
+    y2 <- rep.int(y, elementNROWS(x))
     ## 'unlisted_ans' parallel to 'x'.
     unlisted_ans <- .pintersect_GRanges_GRanges(unlisted_x, y2,
                                         drop.nohit.ranges=FALSE,
@@ -341,28 +383,13 @@ setMethod("pintersect", c("GRanges", "GRangesList"),
                                         strict.strand=strict.strand)
 )
 
-### Equivalent to 'mendoapply(intersect, x, y)'.
-setMethod("pintersect", c("GRangesList", "GRangesList"),
-    function(x, y, ...)
-    {
-        if (length(x) != length(y))
-            stop("'x' and 'y' must have the same length")
-        seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
-        seqlevels(y) <- seqlevels(x)
-        xgr <- deconstructGRLintoGR(x)
-        ygr <- deconstructGRLintoGR(y)
-        gr <- intersect(xgr, ygr, ...)
-        reconstructGRLfromGR(gr, x)
-    }
-)
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### psetdiff()
 ###
 
 setMethod("psetdiff", c("GRanges", "GRanges"),
-    function(x, y, ignore.strand=FALSE, ...)
+    function(x, y, ignore.strand=FALSE)
     {
         if (length(x) != length(y)) 
             stop("'x' and 'y' must have the same length")
@@ -394,15 +421,15 @@ setMethod("psetdiff", c("GRanges", "GRanges"),
 ### TODO: Review the semantic of this method (see previous TODO's for
 ### "punion" and "pintersect" methods for GRanges,GRangesList).
 setMethod("psetdiff", c("GRanges", "GRangesList"),
-    function(x, y, ignore.strand = FALSE, ...)
+    function(x, y, ignore.strand=FALSE)
     {
         ansSeqinfo <- merge(seqinfo(x), seqinfo(y))
         if (length(x) != length(y)) 
             stop("'x' and 'y' must have the same length")
-        ok <- compatibleSeqnames(rep(seqnames(x), elementLengths(y)),
+        ok <- compatibleSeqnames(rep(seqnames(x), elementNROWS(y)),
                                  seqnames(y@unlistData))
         if (!ignore.strand)
-          ok <- ok & compatibleStrand(rep(strand(x), elementLengths(y)),
+          ok <- ok & compatibleStrand(rep(strand(x), elementNROWS(y)),
                                       strand(y@unlistData))
         if (!all(ok)) {
             ok <-
@@ -411,8 +438,8 @@ setMethod("psetdiff", c("GRanges", "GRangesList"),
             y <- y[ok]
         }
         ansRanges <- gaps(ranges(y), start=start(x), end=end(x))
-        ansSeqnames <- rep(seqnames(x), elementLengths(ansRanges))
-        ansStrand <- rep(strand(x), elementLengths(ansRanges))
+        ansSeqnames <- rep(seqnames(x), elementNROWS(ansRanges))
+        ansStrand <- rep(strand(x), elementNROWS(ansRanges))
         ansGRanges <-
           GRanges(ansSeqnames, unlist(ansRanges, use.names=FALSE), ansStrand)
         seqinfo(ansGRanges) <- ansSeqinfo
@@ -420,21 +447,6 @@ setMethod("psetdiff", c("GRanges", "GRangesList"),
     }
 )
 
-### Equivalent to 'mendoapply(setdiff, x, y)'.
-setMethod("psetdiff", c("GRangesList", "GRangesList"),
-    function(x, y, ...)
-    {
-        if (length(x) != length(y))
-            stop("'x' and 'y' must have the same length")
-        seqinfo(x) <- merge(seqinfo(x), seqinfo(y))
-        seqlevels(y) <- seqlevels(x)
-        xgr <- deconstructGRLintoGR(x)
-        ygr <- deconstructGRLintoGR(y)
-        gr <- setdiff(xgr, ygr, ...)
-        reconstructGRLfromGR(gr, x)
-    }
-)
-
 setMethod("pgap", c("GRanges", "GRanges"),
     function(x, y, ignore.strand=FALSE, ...)
     {
@@ -449,7 +461,7 @@ setMethod("pgap", c("GRanges", "GRanges"),
         if (!allCompatibleSeqnamesAndStrand(x, y))
             stop("'x' and 'y' elements must have compatible 'seqnames' ",
                  "and 'strand' values")
-        ranges(x) <- pgap(ranges(x), ranges(y))
+        ranges(x) <- callGeneric(ranges(x), ranges(y), ...)
         x
     }
 )
