@@ -10,7 +10,7 @@ options(width=72)
 ###################################################
 ### code chunk number 2: biocLite (eval = FALSE)
 ###################################################
-## source("http://bioconductor.org/biocLite.R")
+## source("https://bioconductor.org/biocLite.R")
 ## biocLite("GenomicRanges")
 
 
@@ -232,9 +232,9 @@ seqlengths(grl)
 
 
 ###################################################
-### code chunk number 31: elementLengths
+### code chunk number 31: elementNROWS
 ###################################################
-elementLengths(grl)
+elementNROWS(grl)
 
 
 ###################################################
@@ -366,26 +366,8 @@ findOverlaps(grl, gr, select="first")
 
 
 ###################################################
-### code chunk number 50: readGAlignments
+### code chunk number 50: SessionInfo
 ###################################################
-library(GenomicAlignments)
-aln1_file <- system.file("extdata", "ex1.bam", package="Rsamtools")
-aln1 <- readGAlignments(aln1_file)
-aln1
-length(aln1)
-
-
-###################################################
-### code chunk number 51: accessors
-###################################################
-head(seqnames(aln1))
-seqlevels(aln1)
-head(strand(aln1))
-head(cigar(aln1))
-head(qwidth(aln1))
-head(start(aln1))
-head(end(aln1))
-head(width(aln1))
-head(njunc(aln1))
+sessionInfo()
 
 
